;
;    SIMPLE LINUX EMULATOR
;
;    Compile with FASM for Menuet
;
   
use32
   
                org     0x0
   
                db      'MENUET00'              ; 8 byte id
                dd      38                      ; required os
                dd      START                   ; program start
                dd      I_END                   ; program image size
                dd      0x100000                ; required amount of memory
                                                ; esp = 0x7FFF0
                dd      0x00000000              ; reserved=no extended header
   
   
START:                          ; start of execution
   
    call draw_window            ; at first, draw the window
   
still:
   
    mov  eax,5
    mov  ebx,5
    int  0x40
   
    mov  edi,0
    call check_events
   
    jmp  still
   
check_events:
   
    mov  eax,11                 ; check for event
    int  0x40
   
    cmp  eax,1                  ; redraw request ?
    je   red
    cmp  eax,3                  ; button ?
    je   button
   
    cmp  edi,0
    jne  ceret
   
    cmp  eax,2                  ; key ?
     je   key
   
  ceret:
   
    ret
   
   
  red:                          ; redraw
    call draw_window
    jmp  check_events
   
  key:                          ; key
    mov  eax,2                  ; just read it and ignore
    int  0x40
    jmp  check_events
   
  button:                       ; button
    mov  eax,17                 ; get id
    int  0x40
   
    cmp  ah,1                   ; button id=1 ?
    jne  noclose
    mov  eax,-1                 ; close this program
    int  0x40
  noclose:
   
    cmp  ah,2
    jne  no_start_elf
    jmp  start_elf
  no_start_elf:
   
    jmp  check_events
   
   
int_0x80:
   
    pusha
   
    cmp  eax,1               ; end of execution
    jne  no_end
   
    popa
   
    mov  ebx,[xy]
    and  ebx,0xffff
    add  ebx,10*65536+24
    mov  [xy],ebx
   
    mov  esp,0x7f000
   
    jmp  still
   
  no_end:
   
    cmp  eax,3               ; std in
    jne  no_read_stdin
   
    mov  edi,0
   
  wkey:
   
   
    pusha
    mov  edi,1
    call check_events
    popa
   
    mov  eax,5
    mov  ebx,2
    int  0x40
   
    mov  eax,2
    int  0x40
   
    cmp  al,0
    jne  wkey
   
   
    cmp  ah,byte 13
    je   nkey
   
    mov  [ecx],ah
   
    add  ecx,1
    add  edi,1
   
    pusha
   
    mov  eax,4
    mov  ebx,[xy]
    mov  edx,ecx
    dec  edx
    mov  ecx,0x000000
    mov  esi,1
    int  0x40
   
    add  ebx,6*65536
    mov  [xy],ebx
   
    popa
   
    jmp  wkey
   
   nkey:
   
    mov  ebx,[xy]
    and  ebx,0xffff
    add  ebx,10*65536+12
    mov  [xy],ebx
   
    mov  [esp+28],edi
   
    popa
   
    int  0x80
    jmp  int_0x80
   
   
  no_read_stdin:
   
   
    cmp  eax,4               ; std out
    jne  no_write_stdout
   
    mov  edi,edx
   
    mov  edx,ecx
   
   
  newlet:
   
    mov  ebx,[xy]
   
    cmp  [edx],byte 10
    jne  nolf
    and  ebx,0xffff
    add  ebx,4*65536+12
    jmp  yeslf
  nolf:
   
    mov  eax,4
    mov  ecx,0x00000000
    mov  esi,1
    int  0x40
   
  yeslf:
   
    inc  edx
   
    add  ebx,6*65536
    mov  [xy],ebx
   
    dec  edi
    jnz  newlet
   
    popa
   
    int  0x80
    jmp  int_0x80
   
  no_write_stdout:
   
    popa
   
    int  0x80
    jmp  int_0x80
   
   
xy  dd 0x000A0020
   
file_name db 'STDIO      '
   
   
load_elf:
   
    pusha
   
    mov  eax,6
    mov  ebx,file_name
    mov  ecx,0
    mov  edx,-1
    mov  esi,0x80000
    int  0x40
   
    mov  eax,int_0x80
    mov  edi,I_END
    mov  ecx,0x100
    cld
    rep  stosd
   
    mov  eax,49         ; define apps interrupt handlers
    mov  ebx,0
    mov  ecx,I_END
    int  0x40
   
   
    popa
   
    ret
   
   
start_elf:
   
    call load_elf
   
    mov  esi,[0x80000+24]
   
    mov  [0x80000],dword int_0x80
   
    mov  esp,0x7f000
   
    jmp  esi
   
   
   
   
   
   
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
   
   
draw_window:
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40
   
                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,50*65536+400          ; [x start] *65536 + [x size]
    mov  ecx,50*65536+300          ; [y start] *65536 + [y size]
    mov  edx,0x02ffffff            ; color of work area RRGGBB,8->color gl
    mov  esi,0x80556677            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x00556677            ; color of frames    RRGGBB
    int  0x40
   
    mov  eax,8
    mov  ebx,132*65536+140
    mov  ecx,5*65536+12
    mov  edx,2
    mov  esi,0x445566
    int  0x40
                                   ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x00ffffff            ; color of text RRGGBB
    mov  edx,labelt                ; pointer to text beginning
    mov  esi,labellen-labelt       ; text length
    int  0x40
   
                                   ; CLOSE BUTTON
    mov  eax,8                     ; function 8 : define and draw button
    mov  ebx,(400-19)*65536+12     ; [x start] *65536 + [x size]
    mov  ecx,5*65536+12            ; [y start] *65536 + [y size]
    mov  edx,1                     ; button id
    mov  esi,0x667788              ; button color RRGGBB
    int  0x40
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,2                     ; 2, end of draw
    int  0x40
   
    ret
   
   
; DATA AREA
   
labelt:
     db   'SIMPLE LINUX EMU      RUN STDIO -ELF BINARY'
labellen:
   
I_END:
   
   
   
   