; aclock 1.0
; Copyright (c) 2002 Thomas Mathys
; killer@vantage.ch
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


	bits 32
	section .text
	%include 'mos.inc'

	MOS_HEADER01 start,end
	

;**********************************************************
; magic numbers
;**********************************************************

;aclock has some nifty mechanism to read out the day of week
;from the system's rtc. unfortunately, on half of the machines
;aclock was tested on it doesn't work, and so this feature
;is currently deactivated. to activate it, uncomment the
;following line
;;;%define SHOW_DAY_OF_WEEK

WIDTH	equ	112			; window dimensions
HEIGHT	equ	128
XPOS	equ	(640-WIDTH-20)		; from RIGHT of screen
YPOS	equ	20			; from TOP of screen

DATEY	equ	109			; date y position 
%ifdef SHOW_DAY_OF_WEEK
DAYX	equ	((WIDTH - 15*6)/2)	; day x position
MONTHX	equ	(DAYX+4*6)		; month x position
DATEX	equ	(DAYX+8*6)		; date x position
YEARX	equ	(DAYX+11*6)		; year x position
%else
MONTHX	equ	((WIDTH - 11*6)/2)	; month x position
DATEX	equ	(MONTHX+4*6)		; date x position
YEARX	equ	(MONTHX+7*6)		; year x position
%endif

CLOCKCX	equ	(WIDTH/2)		; clock center
CLOCKCY	equ	(HEIGHT/2)

; floating point constants =)
SECR	equ	0x42000000		; second pointer radius (32.0f)
SECD2R	equ	0x3dd67750		; 2*pi/60
MINR	equ	0x41f00000		; minute pointer radius (30.0f)
MIND2R	equ	0x3ae4c389		; 2*pi/60/60
HOURR	equ	0x41800000		; hour pointer radius (16.0f)
HOURD2R	equ	0x3918825b		; 2*pi/60/60/12
TMR1	equ	0x42180000		; tick mark radius #1 (38.0f)
TMR2	equ	0x42100000		; tick mark radius #2 (36.0f)
TMD2R	equ	0x3f060a92		; 2*pi/12
	
	
;**********************************************************
; main program
;**********************************************************

start:
	xor ebx,ebx			; get default window colors
	mov bl,3
	mov ecx,wndColors
	xor edx,edx
	mov dl,MOS_WNDCOLORS_SIZE
	xor eax,eax
	mov al,MOS_SC_WINDOWPROPERTIES
	int 0x40
	call drawWindow			; draw window

.msgpump:
	call drawClock

	xor ebx,ebx			; wait one second or for next event
	mov bl,100
	xor eax,eax
	mov al,MOS_SC_WAITEVENTTIMEOUT
	int 0x40
	
	cmp eax,MOS_EVT_REDRAW		; redraw window ?
	jne .noredraw
	call drawWindow			; yes -> do it
	jmp short .msgpump
.noredraw
	cmp eax,MOS_EVT_KEY		; key in buffer ?
	jne .nokey
	xor eax,eax			; yes -> overread
	mov al,MOS_SC_GETKEY
	int 0x40
	jmp short .msgpump
.nokey
	cmp eax,MOS_EVT_BUTTON		; button pressed ?
	jne .msgpump
	xor eax,eax			; yes -> terminate application
	dec eax
	int 0x40
	
	
;**********************************************************
; define and draw window
;**********************************************************

drawWindow:
	xor ebx,ebx			; start redraw
	inc ebx
	xor eax,eax
	mov al,MOS_SC_REDRAWSTATUS
	int 0x40
	
	mov ebp,wndColors		; accessing the structure via ebp saves space =)	
	
	xor eax,eax			; get screen dimensions
	mov al,MOS_SC_GETSCREENMAX
	int 0x40			; screen dimensions in eax
		
	mov ebx,eax			; calculate x position (ebx bits 24..31)
	sub ebx,(WIDTH+20)*0x10000
	mov bx,WIDTH			; load width
	mov ecx,YPOS*0x10000+HEIGHT	; y position and size

	mov edx,[ebp+MOS_WNDCOLORS.work]
	or edx,0x03000000		; skinned window
	mov esi,[ebp+MOS_WNDCOLORS.grab]
	mov edi,[ebp+MOS_WNDCOLORS.frame]
	xor eax,eax
	int 0x40
	
	call drawClock
	
	xor ebx,ebx			; end redraw
	mov bl,2
	xor eax,eax
	mov al,MOS_SC_REDRAWSTATUS
	int 0x40
	ret

	
drawClock:

	mov ebp,wndColors

	; clear window
	mov ebx,5*0x10000+(WIDTH-10)
	mov ecx,22*0x10000+(HEIGHT-27)
	mov edx,[ebp+MOS_WNDCOLORS.work]
	xor eax,eax
	mov al,MOS_SC_DRAWBAR
	int 0x40
	
	; get system clock
	xor eax,eax
	mov al,MOS_SC_GETSYSCLOCK
	int 0x40
	mov edi,eax			; save in edi
	
	; draw seconds pointer
	shr eax,16			; get seconds
	call bcdbin
	mov esi,eax			; save seconds for later
	push eax
	push dword SECD2R
	push dword SECR
	call getPointerCoords
	add ebx,CLOCKCX*0x10000
	add ecx,CLOCKCY*0x10000
	mov edx,[ebp+MOS_WNDCOLORS.workText]
	xor eax,eax
	mov al,MOS_SC_DRAWLINE
	int 0x40
	
	;draw minutes pointer
	mov eax,edi			; get minutes
	mov al,ah
	call bcdbin
	mov ecx,60
	mul ecx
	add eax,esi			; eax = angle for minutes pointer
	mov esi,eax			; save for later
	push eax
	push dword MIND2R
	push dword MINR
	call getPointerCoords
	add ebx,CLOCKCX*0x10000
	add ecx,CLOCKCY*0x10000
	mov edx,[ebp+MOS_WNDCOLORS.workText]
	xor eax,eax
	mov al,MOS_SC_DRAWLINE
	int 0x40
	
	;draw hours pointer
	mov eax,edi			; get hours
	call bcdbin
	cmp eax,11			; % 12 (theoretically this is not necessary)
	jnae .ok
	sub eax,12
.ok:
	mov ecx,60*60
	mul ecx
	add eax,esi
	push eax
	push dword HOURD2R
	push dword HOURR
	call getPointerCoords
	add ebx,CLOCKCX*0x10000
	add ecx,CLOCKCY*0x10000
	mov edx,[ebp+MOS_WNDCOLORS.workText]
	xor eax,eax
	mov al,MOS_SC_DRAWLINE
	int 0x40
	
	; draw tick marks
	mov esi,11			; esi = loop counter/angle
.drawtickmarks
	push esi			; calculate start point
	push dword TMD2R
	push dword TMR1
	call getPointerCoords
	mov eax,ebx			; save in eax and edx
	mov edx,ecx
	push esi			; calculate end point
	push dword TMD2R
	push dword TMR2
	call getPointerCoords
	shl eax,16
	shl edx,16
	add ebx,eax			; ebx = x start and end
	add ecx,edx			; ecx = y start and end
	mov edx,[ebp+MOS_WNDCOLORS.workText]
	xor eax,eax
	mov al,MOS_SC_DRAWLINE
	int 0x40
	dec esi
	jns .drawtickmarks
	
	; get date
	xor eax,eax
	mov al,MOS_SC_GETDATE
	int 0x40
	mov edi,eax
	
	; display day of week
%ifdef SHOW_DAY_OF_WEEK
	shr eax,24			; get day of week #
	mov ebx,DAYX*0x10000+DATEY	; position
	mov ecx,[ebp+MOS_WNDCOLORS.workText]
	lea edx,[dayNames+eax*2+eax]	; edx -> day name
	mov esi,3			; text length
	xor eax,eax			; load syscall number
	mov al,MOS_SC_WRITETEXT
	int 0x40
%endif	

	; display month
	mov eax,edi			; get month
	mov al,ah
	call bcdbin
	mov ebx,MONTHX*0x10000+DATEY	; position
	mov ecx,[ebp+MOS_WNDCOLORS.workText]
	lea edx,[monthNames-3+eax*2+eax]; edx -> month name (-3 because eax = 1..12 =)
	mov esi,3			; text length
	xor eax,eax
	mov al,MOS_SC_WRITETEXT
	int 0x40
		
	; display date
	mov eax,edi			; get date
	shr eax,16
	call bcdbin
	mov ebx,0x00020000		; number, display two digits
	mov ecx,eax			; number to display
	mov edx,DATEX*0x10000+DATEY
	mov esi,ecx			; color is still in ecx
	xor eax,eax			; load syscall number
	mov al,MOS_SC_WRITENUMBER
	push eax			; save syscall number for later
	int 0x40
	
	; display year
	mov eax,edi			; get year
	call bcdbin
	add eax,2000			; great way to avoid y2k bug :D
	mov ebx,0x00040000		; number, display four digits
	mov ecx,eax			; number to display
	mov edx,YEARX*0x10000+DATEY
	;mov esi,[ebp+MOS_WNDCOLORS.workText] ; color is already in esi
	pop eax				; get syscall number back 
	int 0x40

	ret
	

;**********************************************************
; bcdbin
; converts a 8 bit bcd number into a 32 bit binary number
;
; in		al = 8 bit bcd number
; out		eax = 32 bit binary number
; destroys	dl,flags
;**********************************************************

bcdbin:
	mov dl,al		; save bcd number
	shr al,4		; convert upper nibble
	mov ah,10
	mul ah
	and dl,15		; add lower nibble
	add al,dl
	;and eax,255		; !
	movzx eax,al		; !
	ret
	
;**********************************************************
; getPointerCoords
; calculates the end point of a pointer
;
; in	ANGLE = pointer angle (32 bit real)
; 	DEG2RAD	= conversion factor for angle (32 bit real)
;	RADIUS = pointer radius (32 bit integer)
;
; out	EBX	:	the lower word contains the x
;			coordinate, the upper word is 0
;	ECX	:	the lower word contains the y
;			coordinate, the upper word is 0
;**********************************************************

;pascal void getPointerCoords(int angle,float deg2rad,float radius,int *x,int *y) {

getPointerCoords:

ANGLE	equ	12
DEG2RAD	equ	8
RADIUS	equ	4

	; no, i'm *not* going to use ebp here!

	fild dword [esp+ANGLE]		; get a angle
	fmul dword [esp+DEG2RAD]	; convert to radians
	fsincos
	fmul dword [esp+RADIUS]		; -y * radius + clockcy
	fchs
	fiadd dword [clockcy]
	fistp dword [esp+ANGLE]	
	fmul dword [esp+RADIUS]		; x * radius + clockcx
	fiadd dword [clockcx]
	fistp dword [esp+RADIUS]
	
	mov ebx,[esp+RADIUS]		; upper word = x
	mov ecx,[esp+ANGLE]		; lower word = y

	ret 12


;**********************************************************
; initialized data
;**********************************************************

clockcx	dd	CLOCKCX
clockcy	dd	CLOCKCY

%ifdef SHOW_DAY_OF_WEEK
dayNames:
	db "Sun"
	db "Mon"
	db "Tue"
	db "Wed"
	db "Thu"
	db "Fri"
	db "Sat"
%endif
	
monthNames:
	db "Jan"
	db "Feb"
	db "Mar"
	db "Apr"
	db "May"
	db "Jun"
	db "Jul"
	db "Aug"
	db "Sep"
	db "Oct"
	db "Nov"
	db "Dec"


;**********************************************************
; uninitialized data
;**********************************************************

section .bss

;wndColors istruc MOS_WNDCOLORS		; holds the default window colors
;iend
wndColors resb MOS_WNDCOLORS_SIZE	; get rid of those silly warning messages =)


end:
